import { exec, ExecException } from 'child_process';
import * as fs from 'fs';
import { userConfig } from '../extension';
import { error as err, log } from '../recorder/utils';
import { BASHRC_PATH, ENV_FILE_PATH } from './config';

export default class MetadataProvider {
	public static loadPathsToJson(filePath: string) {
		const portToPath = userConfig.getPaths();
		const serviceNameToPath = this.getServicePathFromPortPath(portToPath);
		const jsonValue = Object.fromEntries(serviceNameToPath);
		const fileContent = JSON.stringify(jsonValue, null, 4) + '\n';
		fs.writeFileSync(filePath, fileContent, { flag: 'w+' });
	}

	public static loadPathsToEnvVariables(): void {
		const portToPath = userConfig.getPaths();
		const serviceNameToPath = this.getServicePathFromPortPath(portToPath);
		let fileContent = '';
		for (const [serviceName, servicePath] of serviceNameToPath.entries()) {
			fileContent += `${serviceName}=${servicePath} \n`;
		}
		fs.writeFileSync(ENV_FILE_PATH, fileContent, { flag: 'w+' });
		const bashrcAppendContent = `
export $(cat ${ENV_FILE_PATH} | xargs -L 1)
`;
		fs.writeFileSync(BASHRC_PATH, bashrcAppendContent, { flag: 'a+' });
		exec(`. ${BASHRC_PATH}`, execCallback);
	}

	/**
	 * two formats to handle: (key="front", value="3001") | (key="8080": value="HXW3fnKwUULrfKS1-cdb")\
	 * expected result: (key="front", value="localhost:3001") | (key="cdb", value="HXW3fnKwUULrfKS1-cdb")
	 *
	 */
	private static getServicePathFromPortPath(portToPath: Map<number, string>): Map<string, string> {
		const serviceToPath = new Map<string, string>();
		for (const [port, path] of Object.entries(portToPath)) {
			let key = '';
			let value = '';
			if (isNumeric(port)) {
				key = path.split('-').slice(-1);
				value = `${path}.${userConfig.getHost()}`;
			} else {
				key = port;
				value = `localhost:${path}`;
			}
			serviceToPath.set(key, value);
		}
		return serviceToPath;
	}
}

function isNumeric(val: string) {
	return /^\d+$/.test(val);
}

function execCallback(error: ExecException | null, stdout: string, stderr: string) {
	if (error) {
		err(`error: ${error.message}`);
		return;
	}
	if (stderr) {
		err(`stderr: ${stderr}`);
		return;
	}
	log(stdout);
}
