import GitMission from '../core/gitMission';
import { PROJECT_SRC_PATH, PROJECT_THEIA_PATH } from '../core/config';
import { format } from 'date-fns';
import { execSync } from 'child_process';
import { existsSync, renameSync, copyFileSync, PathLike } from 'fs';

const util = require('util');
const unlink = util.promisify(require('fs').unlink);
const exec = util.promisify(require('child_process').exec);
const fs = require('fs');
const readdirSync = require('fs').readdirSync;
const Path = require('path');

const PREFIX = '[DEADLOCK-RECORDER]';

export const log = (message: any, args?: any[]) => {
	if (args) {
		console.log(`${PREFIX} ${message}`, args);
	} else {
		console.log(`${PREFIX} ${message}`);
	}
};

export const error = (message: any, args?: any[]) => {
	if (args) {
		console.error(`${PREFIX} ${message}`, args);
	} else {
		console.error(`${PREFIX} ${message}`);
	}
};

export async function copyProjectSources(srcPath: string, theiaPath: string, excludes: Array<string> = []) {
	const excludeCmd = excludes.map((current) => `--exclude ${current}`).join(' ');
	return exec(`rsync -a ${srcPath}/ ${theiaPath} ${excludeCmd} && chown -R theia:theia /home/project`);
}

export async function pathContainsFiles(path: string) {
	return exec(`ls -1 ${path} | wc -l`);
}

export async function clearFilesExceptGit(path) {
	readdirSync(path).forEach(async (file) => {
		if (file !== '.git') {
			const curPath = Path.join(path, file);
			if (fs.lstatSync(curPath).isDirectory()) {
				fs.rmdirSync(curPath, { recursive: true });
			} else {
				await unlink(curPath);
			}
		}
	});
}

function renameIfExistsSync(srcPath: PathLike, destPath: PathLike) {
	if (existsSync(srcPath)) {
		renameSync(srcPath, destPath);
	} else {
		log(`Renaming : No ${srcPath} found`);
	}
}

function copyFileIfExistsSync(srcPath: PathLike, destPath: PathLike) {
	if (existsSync(srcPath)) {
		copyFileSync(srcPath, destPath);
	} else {
		log(`Copying file: No ${srcPath} found`);
	}
}

function copyFolderIfExistsSync(srcPath: PathLike, destPath: PathLike) {
	if (existsSync(srcPath)) {
		execSync(`rsync -r ${srcPath}/* ${destPath}`);
	} else {
		log(`Copying folder: No ${srcPath} found`);
	}
}

/**
 *
 * Rename temporary git files back to actual git files.
 *
 * @param path Path of the folder containing user's temporary git project files.
 * @param userId Identifier used to create the temporary files name
 */
export function renameTempToUserGitFiles(path: string, userId: string) {
	renameIfExistsSync(Path.join(path, `user-git-${userId}`), Path.join(path, '.git'));
	renameIfExistsSync(Path.join(path, `user-gitignore-${userId}`), Path.join(path, '.gitignore'));
}

/**
 *
 * Copy user's git project files and paste them with renamed filenames so that
 * they can be saved by the recorder.
 *
 * @param srcPath Path of the folder containing user's git project files to be saved.
 * @param destPath Path where the user's git project files should be pasted.
 * @param userId Identifier used to create the temporary files name
 */
function copyGitUserFiles(srcPath: string, destPath: string, userId: string) {
	copyFolderIfExistsSync(Path.join(srcPath, '.git'), Path.join(destPath, `user-git-${userId}`));
	copyFileIfExistsSync(Path.join(srcPath, '.gitignore'), Path.join(destPath, `user-gitignore-${userId}`));
}

export enum CommitFrom {
	Run = 'Run',
	Auto = 'Auto',
}

export async function commitAndPushCode(gitMission: GitMission, from: CommitFrom) {
	try {
		log('Commit & push');
		await clearFilesExceptGit(PROJECT_SRC_PATH);

		copyGitUserFiles(PROJECT_THEIA_PATH, PROJECT_SRC_PATH, gitMission.author);
		execSync(
			`rsync -r --exclude .git --exclude npm --exclude target ${PROJECT_THEIA_PATH}/* ${PROJECT_SRC_PATH} && cp ${Path.join(
				__dirname,
				'.gitignore',
			)} ${PROJECT_SRC_PATH} && chown -R root:root /project`,
		);

		const currentDate = format(new Date(), "HH'H'mm'_'dd/LL/y");
		await gitMission.addAll();
		await gitMission.commit(currentDate);
		await gitMission.push();
		log('Commit & push done.');
	} catch (e) {
		error(`[${e.status}] cannot commitAndPush code: ${e.stderr}`);
		error(e.message);
	}
}
