import * as fs from 'fs';
import { marked } from 'marked';
import * as path from 'path';
import { setInterval } from 'timers';
import * as vscode from 'vscode';
import { BRIEFING_FILE_NAME, DOCS_PATH, ENV_FILE_PATH, SERVICES_PATHS_PATH } from '../core/config';
import { userConfig } from '../extension';
import { OPEN_BRIEFING_COMMAND } from '../theia/command';
import { WebviewBase } from './webviewBase';

export const BRIEFING_ID = 'brefingView';
export default class BriefingView extends WebviewBase {
	private briefingContent: string | undefined;

	private errorLoadingBriefing: boolean = false;

	private loadingBriefing: boolean = false;

	constructor() {
		super(BRIEFING_ID, 'Briefing', OPEN_BRIEFING_COMMAND);
	}

	toBase64(file: string) {
		const result = fs.readFileSync(path.join(DOCS_PATH, file), {
			encoding: 'base64',
		});
		return result;
	}

	setupImages(text: string) {
		const imageRegex = /\!\[(.*?)\]\(\b(image:)(.*?\))?/g;

		return text.replace(imageRegex, (match, title, imagePrefix, img) => {
			const fileName = img.replace(')', '');
			const extension = fileName.split('.');

			return `![${title}](data:image/${extension[extension.length - 1]};base64,${this.toBase64(fileName)})`;
		});
	}

	load(): void {
		const intervalId = setInterval(() => {
			if (vscode.workspace && !this.loadingBriefing) {
				this.loadingBriefing = true;
				clearInterval(intervalId);

				vscode.workspace.openTextDocument(vscode.Uri.parse(path.join(DOCS_PATH, BRIEFING_FILE_NAME))).then(
					(document) => {
						try {
							this.briefingContent = this.setupImages(document.getText());
						} catch (e) {
							console.error(e);
							this.briefingContent = 'Error while parsing your briefing.';
						}
						this.show();
					},
					(error) => {
						console.error('Cannot load briefing', error);
						this.errorLoadingBriefing = true;
					},
				);
			}
		}, 500);
	}

	render() {
		let output = '';

		if (userConfig.isProfessor()) {
			output += `
        <h2>Professeur</h2>
        Bonjour ${userConfig.getCurrentUserDetails().lastName} ${
				userConfig.getCurrentUserDetails().firstName
			} vous êtes actuellement entrain de relire le code de <b>${userConfig.getRemoteUserDetails().lastName} ${
				userConfig.getRemoteUserDetails().firstName
			}</b>.<br />
        Vous pouvez consulter chaque exécution de son code dans l'onglet à gauche de GitLens. Chaque commit sur <b>master</b> représente une exécution.
        <br />
      `;
		}

		if (this.briefingContent) {
			output += `<h1>Mission Goal 🕶</h1>${marked(this.briefingContent)}`;
		} else if (this.errorLoadingBriefing) {
			output += 'Cannot load briefing.';
		} else {
			output += 'Loading briefing..';
		}

		output += '<br/>';

		if (userConfig.getUsername()) {
			output += this.renderUserChallengeConfig();
		} else {
			output += 'Loading help..';
		}

		return output;
	}

	private renderUserChallengeConfig() {
		let adresses = '';

		let pathsLength = 0;
		const paths = userConfig.getPaths();
		for (const key in paths) {
			if (key !== '3000') {
				pathsLength++;
				const path = paths[key];
				if (userConfig.getHost() === 'localhost') {
					adresses += `<li>${key} binded on <a href="http://${userConfig.getHost()}:${path}">${path}</a></li>`;
				} else {
					adresses += `<li>${key} binded on <a href="https://${path}.${userConfig.getHost()}">${path}</a></li>`;
				}
			}
		}

		if (pathsLength > 0) {
			return `<h3>Configuration</h3>
      You have the following adresses availables for your challenge : <ul>${adresses}</ul>
      </br>
      You also have access to these paths from:
      <ul>
        <li>
        environment variables (file <b>${ENV_FILE_PATH}</b> already exported)
        </li>
        <li>
        JSON file <b>${SERVICES_PATHS_PATH}</b>
        </li>
      </ul>
      </br>`;
		}

		return '';
	}
}
