import * as path from 'path';
import * as vscode from 'vscode';

function getNonce() {
	let text = '';
	const possible = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
	for (let i = 0; i < 32; i++) {
		text += possible.charAt(Math.floor(Math.random() * possible.length));
	}
	return text;
}

/**
 * @deprecated use #webviewBase instead of
 */
export default abstract class View {
	private static views: Map<string, View> = new Map();

	public currentWebviewPanel: vscode.WebviewPanel | undefined;
	private _disposables: vscode.Disposable[] = [];
	private isRegisteredOnWebviewPanelSerializer: boolean;

	constructor(
		id: string,
		private readonly context: vscode.ExtensionContext,
		public readonly panelName: string,
		public readonly title: string,
	) {
		this.isRegisteredOnWebviewPanelSerializer = false;

		if (View.views.has(id)) {
			console.warn(`View has been already created: ${id}`);
			return;
		}
		View.views.set(id, this);
		this.load();
		context.subscriptions.push(this);
	}

	static getViews(): Map<string, View> {
		return this.views;
	}

	static getView(id: string): View {
		if (!View.views.has(id)) {
			console.error(`View ${id} does not exist`);
		}
		return View.views.get(id)!;
	}

	private init(currentWebviewPanel) {
		// Listen for when the panel is disposed
		// This happens when the user closes the panel or when the panel is closed programatically
		currentWebviewPanel.onDidDispose(() => this.dispose(), null, this._disposables);

		// Update the content based on view changes
		currentWebviewPanel.onDidChangeViewState(
			() => {
				if (this.currentWebviewPanel?.visible) {
					this.update();
				}
			},
			null,
			this._disposables,
		);

		// Handle messages from the webview
		currentWebviewPanel.webview.onDidReceiveMessage(
			(message) => {
				switch (message.command) {
					case 'alert':
						vscode.window.showErrorMessage(message.text);
						return;
				}
			},
			null,
			this._disposables,
		);

		if (vscode.window.registerWebviewPanelSerializer && !this.isRegisteredOnWebviewPanelSerializer) {
			// Make sure we register a serializer in activation event
			vscode.window.registerWebviewPanelSerializer(this.panelName, {
				async deserializeWebviewPanel(webviewPanel: vscode.WebviewPanel, state: any) {
					//   View.revive(webviewPanel, extensionPath);
				},
			});
			this.isRegisteredOnWebviewPanelSerializer = true;
		}
	}

	public createOrShow() {
		const column = vscode.window.activeTextEditor ? vscode.window.activeTextEditor.viewColumn : undefined;

		// If we already have a panel, show it.
		if (this.currentWebviewPanel) {
			this.currentWebviewPanel.reveal(column);
			return this.currentWebviewPanel;
		}

		// Otherwise, create a new panel.
		this.currentWebviewPanel = vscode.window.createWebviewPanel(
			this.panelName,
			this.title,
			column || vscode.ViewColumn.One,
			{
				// Enable javascript in the webview
				enableScripts: true,

				// And restrict the webview to only loading content from our extension's `media` directory.
				// for now we don't need to include the media folder, because React bundle is not used
				// localResourceRoots: [
				// vscode.Uri.file(path.join(this.extensionPath, 'media')),
				// ],
			},
		);
		// update first time to force render of webviewPanel
		this.update();
		this.init(this.currentWebviewPanel);
		this.update();
		return this.currentWebviewPanel;
	}

	public doRefactor() {
		// Send a message to the webview webview.
		// You can send any JSON serializable data.
		this.currentWebviewPanel?.webview.postMessage({ command: 'refactor' });
	}

	public dispose() {
		// Clean up our resources
		this.currentWebviewPanel?.dispose();
		this.currentWebviewPanel = undefined;

		while (this._disposables.length) {
			const x = this._disposables.pop();
			if (x) {
				x.dispose();
			}
		}
	}

	update() {
		if (this.currentWebviewPanel) {
			const webview = this.currentWebviewPanel.webview;
			this.currentWebviewPanel.title = this.title;
			webview.html = this._getHtmlForWebview(webview);
		} else {
			console.warn(`Cannot update ${this.panelName} has not been created.`);
		}
	}

	private _getHtmlForWebview(webview: vscode.Webview) {
		// /!\ EXPERIMENTAL /!\
		// Local path to main script run in the webview
		// const scriptPathOnDisk = vscode.Uri.file(
		// path.join(this.extensionPath, 'media', 'static/bundle.js')
		// );
		// const cssPathOnDisk = vscode.Uri.file(
		// path.join(this.extensionPath, 'media', 'static/bundle.css')
		// );

		// And the uri we use to load this script in the webview
		// const scriptUri = webview.asWebviewUri(scriptPathOnDisk);
		// const cssUri = webview.asWebviewUri(cssPathOnDisk);
		// append in the HTML head <link href="${cssUri}" rel="stylesheet"></head>

		// Use a nonce to whitelist which scripts can be run
		// const nonce = getNonce();

		// append in the HTML head<script nonce="${nonce}" src="${scriptUri}"></script>

		let content = '';
		try {
			content = this.render();
		} catch (e) {
			console.error('Erreur lors du render', e);
			content = "Impossible d'afficher le rendu, nous sommes désolés";
		}

		return `<!DOCTYPE html><html lang="en">
		<head>
		  <meta charset="UTF-8">
		  <meta name="viewport" content="width=device-width, initial-scale=1.0">
		  <title>Deadlock</title>
		</head>
		<body>
			<div>
				${content}
			</div>
		</body></html>`;
	}

	/**
	 * Must return the content displayed within the webViewPanel
	 */
	abstract render(): string;
	/**
	 * Method called at the end of first render
	 */
	abstract load(): void;
}
